using System;
using System.Collections;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

using DataProvider;

using Syncfusion.Windows.Forms.Grid;

namespace VirtualGridDemo
{
    public partial class frmMain : Form
    {
        private VirtualDataSource gridDataSource;
        DefaultLists defaultLists;

        public frmMain()
        {
            InitializeComponent();
        }

        private void frmMain_Load(object sender, EventArgs e)
        {
            this.gridDataSource = new VirtualDataSource(4);
            this.defaultLists = new DefaultLists();

            //Resetting grid properties
            Grid.ResetVolatileData();

            //Telling the grid about the number of rows and columns
            Grid.RowCount = gridDataSource.RowCount;
            Grid.ColCount = gridDataSource.ColCount;

            //Hooking up events necessary for virtual grid
            Grid.QueryRowCount +=new Syncfusion.Windows.Forms.Grid.GridRowColCountEventHandler(Grid_QueryRowCount);
            Grid.QueryColCount +=new Syncfusion.Windows.Forms.Grid.GridRowColCountEventHandler(Grid_QueryColCount);
            Grid.QueryCellInfo +=new Syncfusion.Windows.Forms.Grid.GridQueryCellInfoEventHandler(Grid_QueryCellInfo);

            //Hooking up SaveCellInfo event to save the values in the datasource
            Grid.SaveCellInfo +=new GridSaveCellInfoEventHandler(Grid_SaveCellInfo);

            Grid.Tag = this.gridDataSource;

        }

        void Grid_QueryRowCount(object sender, GridRowColCountEventArgs e)
        {
            e.Count = ((VirtualDataSource)Grid.Tag).RowCount;
            e.Handled = true;
        }

        void Grid_QueryColCount(object sender, GridRowColCountEventArgs e)
        {
            e.Count = ((VirtualDataSource)Grid.Tag).ColCount;
            e.Handled = true;
        }

        void Grid_QueryCellInfo(object sender, GridQueryCellInfoEventArgs e)
        {
            #region [setting columns headers]
            if (e.RowIndex == 0)
            {
                switch (e.ColIndex)
                {
                    case 1:
                        e.Style.Text = "Genie Objects";
                        break;

                    case 2:
                        e.Style.Text = "Operators";
                        break;

                    case 3:
                        e.Style.Text = "Values";
                        break;

                    case 4:
                        e.Style.Text = "Operators";
                        break;
                    
                    default:
                        break;
                }
            }
            #endregion
            #region [setting other properties]
            if (e.RowIndex > 0 && e.ColIndex > 0)
            {
                #region [setting cell types]
                if (e.ColIndex > 0 && (e.ColIndex <= 2 || e.ColIndex == 4))
                {
                    e.Style.CellType = "ComboBox";
                }
                #endregion
                #region [setting default values]
                switch (e.ColIndex)
                {
                    case 1:
                        e.Style.DataSource = this.defaultLists.GenieObjects;
                        break;

                    case 2:
                        e.Style.DataSource = this.defaultLists.ComparisonOperators;
                        break;

                    case 4:
                        e.Style.DataSource = this.defaultLists.LogicalOperators;
                        break;

                    default:
                        break;
                }
                #endregion
                #region [loading values from data source]
                RowObject currentRowObject = (RowObject)((VirtualDataSource)Grid.Tag).RowObjects[e.RowIndex - 1];
                RowElement currentRowElement = (RowElement)currentRowObject.RowElements[e.ColIndex - 1];
                e.Style.Text = currentRowElement.RowElementData;
                #endregion
            }
            #endregion
        }

        void Grid_SaveCellInfo(object sender, GridSaveCellInfoEventArgs e)
        {
            try
            {
                if (e.Style.Text == "OR" || e.Style.Text == "AND")
                {
                    //Adding new row object to the gridDataSource
                    this.gridDataSource.AddNewRowObject();

                    //Resetting grid properties again
                    this.Grid.ResetVolatileData();
                    
                    //Telling the grid about the number of rows and columns again
                    this.Grid.RowCount = gridDataSource.RowCount;
                    this.Grid.ColCount = gridDataSource.ColCount;

                    //Refreshing the grid to show the newly added row
                    this.Grid.RefreshRange(GridRangeInfo.Row(e.RowIndex + 1));
                }

                //Getting current row-object
                RowObject currentRowObject = (RowObject)((VirtualDataSource)Grid.Tag).RowObjects[e.RowIndex - 1];

                //Saving cell value into the row-element in the row object
                ((RowElement)currentRowObject.RowElements[e.ColIndex - 1]).RowElementData = e.Style.Text;
            }
            catch (Exception ex)
            {
                throw ex;
            }
        }

    }
}